#pragma warning (disable: 4995)
#pragma warning (disable: 4996)
#include <tchar.h>
#include <windows.h>
#include <objbase.h>

#include <map>
#include <stdexcept>

#include "plugin.hpp"

#include "farManager.h"
#include "farEditor.h"
#include "exception.h"
#include "countof.h"
#include "editorContext.h"
#include "language.h"


#ifndef UNICODE
#define EXP_NAME(p) _export p
#else
#define EXP_NAME(p) _export p ## W
#endif


bool needUninitialize;
std::map<int, EditorContext> *editors;


int __cdecl main() {}

void WINAPI EXP_NAME(SetStartupInfo)(const PluginStartupInfo* Info)
{
	if (editors==NULL) {
		editors = new std::map<int, EditorContext>;
		lng::init();
	}
	farManager = *Info;
	needUninitialize = false;
	switch (HRESULT hr = CoInitialize(0))
	{
	case S_OK: // success
	case S_FALSE: // success -- A different plugin has already
		// initialized COM on this thread in single-thread
		// apartment mode
		needUninitialize = true;
	case RPC_E_CHANGED_MODE: // success -- A different plugin
		// has already initialized COM on this thread in
		// multithread apartment mode
		return;
	default:
		ASSERT_SUCCESS(hr, "Cannot initialize COM");
	}
}

void WINAPI EXP_NAME(GetPluginInfo)(PluginInfo* Info)
{
	static const TCHAR pluginName[] = _T("SSA Editor");
	static const TCHAR* const pluginCommands[] = {
		pluginName
	};

	Info->StructSize = sizeof(PluginInfo);
	Info->Flags = PF_DISABLEPANELS | PF_EDITOR;
	Info->DiskMenuStrings = 0;
	Info->DiskMenuNumbers = 0;
	Info->DiskMenuStringsNumber = 0;
	Info->PluginMenuStrings = pluginCommands;
	Info->PluginMenuStringsNumber = COUNTOF(pluginCommands);
	Info->PluginConfigStrings = 0;
	Info->PluginConfigStringsNumber = 0;
	Info->CommandPrefix = 0;
	Info->Reserved = 0;
}

HANDLE WINAPI EXP_NAME(OpenPlugin)(int OpenFrom, int Item)
{
	try
	{
		(*editors)[GetEditorInfo().EditorID].mainMenu();
	}
	catch (const std::exception& e)
	{
		farManager.showError(e);
	}

	return INVALID_HANDLE_VALUE;
}

void WINAPI EXP_NAME(ExitFAR)()
{
	editors->clear();
	lng::finalize();
	delete editors;
	if (needUninitialize)
		CoUninitialize();
}

int WINAPI EXP_NAME(ProcessEditorEvent)(int Event, void* Param)
{
	try
	{
		switch (Event)
		{
		case EE_CLOSE:
			editors->erase(*reinterpret_cast<int*>(Param));
			break;
		};
	}
	catch (const std::exception& e)
	{
		farManager.showError(e);
	}
	return 0;
}

int WINAPI EXP_NAME(ProcessEditorInput)(const INPUT_RECORD* rec)
{
	try
	{
		return (*editors)[GetEditorInfo().EditorID].processEditorInput(*rec);
	}
	catch (const std::exception& e)
	{
		farManager.showError(e);
	}
	return 0;
}
