#ifndef FAR_EDITOR_INCLUDED
#define FAR_EDITOR_INCLUDED

#include "plugin.hpp"

const int DEFAULT = -1;
const int CURRENT_LINE = -1;
const int END_LINE = -1;

enum SubFileFormat {
	FF_SSA,
	FF_ASS
};

class GetEditorInfo : public EditorInfo
{
public:
	GetEditorInfo();
};

class EditorPositionSaver : public EditorInfo
{
public:
	EditorPositionSaver();
	~EditorPositionSaver();
	void commit();
	void revert();
private:
	bool committed_;
};

class EditorLine
{
public:
	explicit EditorLine(const EditorGetString& getString);

	typedef const TCHAR* const_iterator;

	const_iterator begin() const;
	const_iterator end() const;
	size_t size() const;

	const_iterator eol() const;

	bool selected() const;

	void set(const TCHAR* begin, size_t size);

private:
	int lineNumber_;
	const TCHAR* const begin_;
	const TCHAR* const eol_;
	const size_t size_;
	const int selStart_, selEnd_;
};

extern
class FarEditor
{
private:
	bool dirty_;
public:
	void invalidate();
	void redraw();

	EditorLine currentLine();
	enum SubFileFormat fileFormat;

	template <typename Predicate>
	bool findLineIf(int startLine, int endLine, Predicate predicate);

	template <typename Functor>
	void forEachLine(int startLine, int endLine, Functor functor);

#ifndef _UNICODE
	void convertToOem(TCHAR* text, size_t size);
	void convertFromOem(TCHAR* text, size_t size);
#endif

	void setPosition(int line = DEFAULT, int pos = DEFAULT, int tabPos = DEFAULT,
		int topLine = DEFAULT, int leftPos = DEFAULT, int overtype = DEFAULT);
	void insertLine();
	void insertText(const TCHAR* text);
	void deleteLine();

	void save();

	void startUndoBlock();
	void finishUndoBlock();
} farEditor;


#include "farEditor.inline.h"

#endif
