#ifndef FAR_MANAGER_INCLUDED
#define FAR_MANAGER_INCLUDED

#include "plugin.hpp"

#include <exception>
#include <string>
#include <sstream>
#include <vector>

class LanguageString
{
public:
	explicit LanguageString(int id);

	const TCHAR* get() const;
	operator const TCHAR*() const;
private:
	int id_;
};

FarMenuItem menuItem(const TCHAR* title, bool selected, bool checked);

const FarMenuItem menuSeparator = {
	_T(""), 0, 0, 1
};

class EnableIf
{
public:
	explicit EnableIf(bool condition);

	void operator()(FarMenuItemEx& item) const;
private:
	bool condition_;
};

class CheckIf
{
public:
	explicit CheckIf(bool condition);

	void operator()(FarMenuItem& item) const;
	void operator()(FarMenuItemEx& item) const;
private:
	bool condition_;
};

class SelectIf
{
public:
	explicit SelectIf(bool condition);

	void operator()(FarMenuItem& item) const;
	void operator()(FarMenuItemEx& item) const;
private:
	bool condition_;
};


class FarMenuItems : public std::vector<FarMenuItem>
{
public:
	FarMenuItems& operator()(const TCHAR* title);
	FarMenuItems& operator()(LanguageString title);
	FarMenuItems& operator()();
	template <typename Functor>
	FarMenuItems& operator()(const Functor& functor);
};

class FarMenuItemsEx : public std::vector<FarMenuItemEx>
{
public:
	FarMenuItemsEx& operator()(const TCHAR* title);
	FarMenuItemsEx& operator()(LanguageString title);
	FarMenuItemsEx& operator()();
	template <typename Functor>
	FarMenuItemsEx& operator()(const Functor& functor);
};

extern
class FarManager : private PluginStartupInfo
{
public:
	FarManager& operator=(const PluginStartupInfo& other);

	const TCHAR* moduleName() const;

	int editorControl(int command, void* param);

	int inputBox(const TCHAR* title, const TCHAR* prompt, const TCHAR* history,
		const TCHAR* srcText, TCHAR* destText, int destLength,
		const TCHAR* help,
		DWORD flags);

	int menu(const FarMenuItem* items, size_t itemCount,
		const TCHAR* title = 0,
		const TCHAR* bottom = 0,
		const TCHAR* help = 0,
		const int* breakKeys = 0,
		int* breakCode = 0);
	int menu(const FarMenuItems& items,
		const TCHAR* title = 0,
		const TCHAR* bottom = 0,
		const TCHAR* help = 0,
		const int* breakKeys = 0,
		int* breakCode = 0);
	int menu(const FarMenuItemEx* items, size_t itemCount,
		const TCHAR* title = 0,
		const TCHAR* bottom = 0,
		const TCHAR* help = 0,
		const int* breakKeys = 0,
		int* breakCode = 0);
	int menu(const FarMenuItemsEx& items,
		const TCHAR* title = 0,
		const TCHAR* bottom = 0,
		const TCHAR* help = 0,
		const int* breakKeys = 0,
		int* breakCode = 0);
	int dialog(FarDialogItem* items, size_t itemCount,
		int width, int height,
		const TCHAR* help = 0);
	int dialogEx(FarDialogItem* items, size_t itemCount,
		int width, INT height,
		const TCHAR* help,
		FARWINDOWPROC dlgProc);

	HANDLE dialogInit(FarDialogItem* items, size_t itemCount,
		int width, int height,
		const TCHAR* help = 0);
	int dialogRun(HANDLE hDlg);
	void dialogFree(HANDLE hDlg);

	long setTextPtr(HANDLE hDlg, int itemId, const TCHAR* text);
	long defDlgProc(HANDLE hDlg, int Msg, int Param1, long Param2);

	const TCHAR* getMsg(int id) const;

	void showMessage(const TCHAR* text, const TCHAR* title = _T("")) const;
	void showMessage(const std::string& s, const TCHAR* title = _T("")) const;
	void showMessage(const std::ostringstream& oss, const TCHAR* title = _T("")) const;
	void showErrorMsg(const TCHAR* text) const;
	void showError(const std::exception& e) const;
	LONG_PTR sendDlgMessage(HANDLE hDlg, int Msg, int Param1, LONG_PTR Param2);

} farManager;

#include "farManager.inline.h"

#endif