#ifndef FAR_MANAGER_INLINE_INCLUDED
#define FAR_MANAGER_INLINE_INCLUDED

#include <cstring>

#include "plugin.hpp"
#include "countof.h"

inline
FarMenuItem
menuItem(const TCHAR* title, bool selected, bool checked)
{
	FarMenuItem result = {
		_T(""),
		selected ? 1 : 0,
		checked ? 1 : 0,
		0
	};
#ifndef _UNICODE
	std::strncpy(result.Text, title, COUNTOF(result.Text));
#else
	result.Text = title;
#endif
	return result;
}

inline
EnableIf::EnableIf(bool condition)
: condition_(condition)
{
}

inline
void
EnableIf::operator()(FarMenuItemEx& item) const
{
	item.Flags |= condition_ ? 0 : MIF_DISABLE;
}

inline
CheckIf::CheckIf(bool condition)
: condition_(condition)
{
}

inline
void
CheckIf::operator()(FarMenuItem& item) const
{
	item.Checked = condition_ ? 1 : 0;
}

inline
void
CheckIf::operator()(FarMenuItemEx& item) const
{
	item.Flags |= condition_ ? MIF_CHECKED : 0;
}

inline
SelectIf::SelectIf(bool condition)
: condition_(condition)
{
}

inline
void
SelectIf::operator()(FarMenuItem& item) const
{
	item.Checked = condition_ ? 7 : 0;
	item.Selected = condition_ ? 1 : 0;
}

inline
void
SelectIf::operator()(FarMenuItemEx& item) const
{
	item.Flags |= condition_ ? (MIF_SELECTED | 7) : 0;
}

inline
FarMenuItems&
FarMenuItems::operator()(const TCHAR* title)
{
	resize(size() + 1);
#ifndef _UNICODE
	std::strncpy(back().Text, title, COUNTOF(back().Text));
#else
	back().Text = title;
#endif;
	back().Selected = 0;
	back().Checked = 0;
	back().Separator = 0;
	return *this;
}

inline
FarMenuItems&
FarMenuItems::operator()(LanguageString title)
{
	return (*this)(title.get());
}

inline
FarMenuItems&
FarMenuItems::operator()()
{
	push_back(menuSeparator);
	return *this;
}

template <typename Functor>
FarMenuItems&
FarMenuItems::operator()(const Functor& functor)
{
	functor(back());
	return *this;
}

inline
FarMenuItemsEx&
FarMenuItemsEx::operator()(const TCHAR* title)
{
	resize(size() + 1);
	back().Flags = 0;
#ifndef _UNICODE
	std::strncpy(&back().Text.Text[0], title, COUNTOF(back().Text.Text));
#else
	back().Text = title;
#endif;
	back().AccelKey = 0;
	back().Reserved = 0;
	back().UserData = 0;
	return *this;
}

inline
FarMenuItemsEx&
FarMenuItemsEx::operator()(LanguageString title)
{
	return (*this)(title.get());
}

inline
FarMenuItemsEx&
FarMenuItemsEx::operator()()
{
	resize(size() + 1);
	back().Flags = MIF_SEPARATOR;
#ifndef _UNICODE
	back().Text.Text[0] = 0;
#else
	back().Text = L"";
#endif;
	return *this;
}

template <typename Functor>
FarMenuItemsEx&
FarMenuItemsEx::operator()(const Functor& functor)
{
	functor(back());
	return *this;
}

inline
LanguageString::LanguageString(int id)
: id_(id)
{
}

inline
LanguageString::operator const TCHAR*() const
{
	return get();
}

inline
const TCHAR* LanguageString::get() const
{
	return farManager.getMsg(id_);
}

inline
FarManager&
FarManager::operator=(const PluginStartupInfo& other)
{
	PluginStartupInfo::operator=(other);
	return *this;
}

inline
const TCHAR*
FarManager::moduleName() const
{
	return ModuleName;
}

inline
int
FarManager::editorControl(int command, void* param)
{
	return EditorControl(command, param);
}

inline
int
FarManager::inputBox(const TCHAR* title, const TCHAR* prompt, const TCHAR* history,
	const TCHAR* srcText, TCHAR* destText, int destLength,
	const TCHAR* help,
	DWORD flags)
{
	return InputBox(title, prompt, history,
		srcText, destText, destLength,
		help,
		flags);
}

inline
int
FarManager::menu(const FarMenuItem* items, size_t itemCount,
	const TCHAR* title,
	const TCHAR* bottom,
	const TCHAR* help,
	const int* breakKeys,
	int* breakCode)
{
	return Menu(ModuleNumber,
		-1, -1, 0,
		FMENU_WRAPMODE,
		title,
		bottom,
		help,
		breakKeys, breakCode,
		items, itemCount);
}

inline
int
FarManager::menu(const FarMenuItems& items,
	const TCHAR* title,
	const TCHAR* bottom,
	const TCHAR* help,
	const int* breakKeys,
	int* breakCode)
{
	return menu(&items[0], items.size(), title, bottom, help, breakKeys, breakCode);
}

inline
int
FarManager::menu(const FarMenuItemEx* items, size_t itemCount,
	const TCHAR* title,
	const TCHAR* bottom,
	const TCHAR* help,
	const int* breakKeys,
	int* breakCode)
{
	return Menu(ModuleNumber,
		-1, -1, 0,
		FMENU_WRAPMODE | FMENU_USEEXT,
		title,
		bottom,
		help,
		breakKeys, breakCode,
		reinterpret_cast<const FarMenuItem*>(items), itemCount);
}

inline
int
FarManager::menu(const FarMenuItemsEx& items,
	const TCHAR* title,
	const TCHAR* bottom,
	const TCHAR* help,
	const int* breakKeys,
	int* breakCode)
{
	return menu(&items[0], items.size(), title, bottom, help, breakKeys, breakCode);
}

inline
int
FarManager::dialog(FarDialogItem* items, size_t itemCount,
	int width, int height,
	const TCHAR* help)
{
#ifndef _UNICODE
	return Dialog(ModuleNumber,
		-1, -1, width, height,
		help,
		items, itemCount);
#else
	HANDLE hDlg = DialogInit(ModuleNumber,
		-1, -1, width, height,
		help,
		items, itemCount, 0, 0, NULL, NULL);
	int res = DialogRun(hDlg);
	DialogFree(hDlg);
	return res;
#endif
}

inline
int
FarManager::dialogEx(FarDialogItem* items, size_t itemCount,
	int width, int height,
	const TCHAR* help,
	FARWINDOWPROC dlgProc)
{
#ifndef _UNICODE
	return DialogEx(ModuleNumber,
		-1, -1, width, height,
		help,
		items, itemCount,
		0, // Reserved
		0, // Flags
		dlgProc,
		0);
#else
	HANDLE hDlg = DialogInit(ModuleNumber,
		-1, -1, width, height,
		help,
		items, itemCount, 0, 0, dlgProc, NULL);
	int res = DialogRun(hDlg);
	DialogFree(hDlg);
	return res;
#endif
}

inline
HANDLE FarManager::dialogInit(FarDialogItem* items, size_t itemCount,
		int width, int height,
		const TCHAR* help)
{
	return DialogInit(ModuleNumber,
		-1, -1, width, height,
		help,
		items, itemCount, 0, 0, NULL, NULL);
}
inline
int FarManager::dialogRun(HANDLE hDlg)
{
	return DialogRun(hDlg);
}

inline
void FarManager::dialogFree(HANDLE hDlg)
{
	return DialogFree(hDlg);
}



inline
long
FarManager::setTextPtr(HANDLE hDlg, int itemId, const TCHAR* text)
{
	return SendDlgMessage(hDlg, DM_SETTEXTPTR, itemId, reinterpret_cast<long>(text));
}

inline
long
FarManager::defDlgProc(HANDLE hDlg, int Msg, int Param1, long Param2)
{
	return DefDlgProc(hDlg, Msg, Param1, Param2);
}

inline
const TCHAR*
FarManager::getMsg(int id) const
{
	return GetMsg(ModuleNumber, id);
}

inline
void
FarManager::showMessage(const TCHAR* text, const TCHAR* title) const
{
	const TCHAR* items[2];
	items[0] = title;
	items[1] = text;
	Message(ModuleNumber,
		FMSG_MB_OK, 0, items, 2, 0);
}

inline
void
FarManager::showMessage(const std::string& s, const TCHAR* title) const
{
	return showMessage(s.c_str(), title);
}

inline
void
FarManager::showMessage(const std::ostringstream& oss, const TCHAR* title) const
{
	return showMessage(oss.str().c_str(), title);
}

inline
void
FarManager::showErrorMsg(const TCHAR* text) const
{
	const TCHAR* items[2];
	items[0] = _T("Error");
	items[1] = text;
	Message(ModuleNumber,
		FMSG_WARNING | FMSG_MB_OK, 0, items, 2, 0);
}

inline
void
FarManager::showError(const std::exception& e) const
{
#ifndef _UNICODE
	showErrorMsg(e.what());
#else
	TCHAR what[1024];
	int nsym = MultiByteToWideChar(CP_ACP,0,e.what(),strlen(e.what()),what,COUNTOF(what));
	if (nsym>=COUNTOF(what)) nsym = COUNTOF(what)-1;
	what[nsym] = 0;
	showErrorMsg(what);
#endif
}

inline
LONG_PTR
FarManager::sendDlgMessage(HANDLE hDlg, int Msg, int Param1, LONG_PTR Param2)
{
	return SendDlgMessage(hDlg, Msg, Param1, Param2);
}

#endif